<?php

/**
 *
 */
function commerce_priceminister_feeds_listing() {

  $header = array(
    t('Title'),
    t('Status'),
    t('Operations'),
  );

  $feeds = entity_load_multiple_by_name('commerce_priceminister_feed');

  $rows = array();
  foreach ($feeds as $feed) {
    // Operations.
    $operations = array(
      l(t('Reports'), 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/reports'),
      l(t('Edit'), 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/edit'),
      l(t('Mapping'), 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/map'),
      l(t('Products'), 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/product'),
      l(t('Export'), 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/export'),
    );

    $rows[] = array(
      'data' => array(
        'title' => $feed->title,
        'status' => ($feed->status) ?  t('Enable'): t('Disable'),
        'operations' => implode(' | ', $operations),
      ),
    );
  }

  $content['link'] = array(
    '#type' => 'item',
    '#markup' => l(t('Add a feed'), 'admin/commerce/config/marketplaces/priceminister/feed/add'),
  );

  $content['information'] = array(
    '#markup' => '<p>' . t('In order to map your products with Priceminister you need to create a feed of products. Each feed will be mapped with a category of Priceminister.') . '</p>'
  );

  // Create the page.
  $content['pager_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('Your products are not ready to be exported. You have to create a <a href="@feed-link">new feed</a> to start sending them to Priceminister.', array('@feed-link' => url('admin/commerce/config/marketplaces/priceminister/feed/add'))),
  );

  // Attach the pager theme.
  $content['pager_pager'] = array('#theme' => 'pager');

  if (!$commerce_priceminister_api = variable_get('commerce_priceminister_api', array())) {
    drupal_set_message(t('The module is not configured yet to communicate with Priceminister. Got to the <a href="@admin-page">administration page</a> to fill in your credentials.', array('@admin-page' => url('admin/commerce/config/marketplaces/priceminister/configure'))), 'warning');
  }

  return $content;
}

/**
 * Page callback: List all reports gathered from Priceminister.
 *
 * @param CommercePriceministerFeed $feed
 * @return array
 */
function commerce_priceminister_feeds_overview(CommercePriceministerFeed $feed) {
  module_load_include('inc', 'commerce_priceminister', 'includes/commerce_priceminister.export');

  $api = new PriceministerAPI();
  commerce_priceminister_check_export_batch($feed, $api);

  $result = $rows = $content = array();

  $header = array(
    array('data' => t('Status')),
    array('data' => t('Success rate')),
    array('data' => t('Total products')),
    array('data' => t('Exported products')),
    array('data' => t('Error products')),
    array('data' => t('Description')),
    array('data' => t('Created'), 'field' => 'w.wid', 'sort' => 'desc'),
    array('data' => t('Processed')),
    array('data' => t('Details')),
  );

  $query = db_select('commerce_priceminister_report_feed', 'cprf')->extend('PagerDefault');
  $query->fields('cprf');
  $query->condition('cprf.feed_id', $feed->id);
  $query->limit(20);
  $query->orderBy('created', 'DESC');
  $result = $query->execute();

  foreach ($result as $record) {

    $upload_date =  (!empty($record->uploaded))
    ? format_date($record->uploaded, 'short')
    : $upload_date = NULL;

    $process_date = (!empty($record->processed))
      ? format_date($record->processed, 'short')
      : t('Not processed yet');

    // Hides the details link if no error occured for this export
    $detail_link = ($record->total_products == $record->exported_products && $record->error_products == 0)
      ? ''
      : l(t('voir details'), 'admin/commerce/config/marketplaces/priceminister/feed/report/' . $record->id . '/details');

    $rows[] = array(
      'data' => array(
        //array('class' => 'icon'),
        $record->status,
        $record->success_rate,
        $record->total_products,
        $record->exported_products,
        $record->error_products,
        $record->description,
        $upload_date,
        $process_date,
        $detail_link,
      ),
    );
  }

  // Create the page.
  $content['pager_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('There are no feed found in the database'),
  );

  $content['pager_pager'] = array('#theme' => 'pager');

  return $content;
}

function commerce_priceminister_feeds_overview_detail($report_id) {
  $content = '';
  // Ensure we have a valid product SKU.
  if (is_numeric($report_id)) {
    $result = $rows = $content = array();

    $header = array(
      array('data' => t('SKU')),
      array('data' => t('Error description')),
      array('data' => t('Error field')),
    );

    $query = db_select('commerce_priceminister_report_product', 'cprp')->extend('PagerDefault');
    $query->fields('cprp');
    $query->condition('cprp.report_id', $report_id);
    $query->limit(20);
    $query->orderBy('created', 'DESC');
    $result = $query->execute();

    foreach ($result as $record) {
      if ($data = unserialize($record->data)) {
        $row[] = array('data' => $record->sku, 'rowspan' => count($data));
        foreach ($data as $error) {
          $row[] = $error['message'];
          $row[] = $error['field'];
          $rows[] = array('data' => $row);
          $row = array();
        }
      }
    }

    // Create the page.
    $content['pager_table'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t("There are no details associated with this report."),
    );

    $content['pager_pager'] = array('#theme' => 'pager');
  }
  else {
    drupal_set_message(t('A report with ID %report_id could not be found.', array('%report_id' => $report_id)), 'error');
  }
  return $content;
}

function commerce_priceminister_product_overview_detail($product_sku) {
  $content = '';
  // Ensure we have a valid product SKU.
  if (($product = commerce_product_load_by_sku($product_sku)) && (is_object($product))) {
    $rows = $content = array();

    $header = array(
      array('data' => t('SKU')),
      array('data' => t('Error')),
      array('data' => t('Description')),
    );

    $query = db_select('commerce_priceminister_report_product', 'cprp');
    $query->fields('cprp');
    $query->condition('cprp.sku', $product_sku);
    $query->range(0, 1);
    $query->orderBy('created', 'DESC');
    $result = $query->execute();

    foreach ($result as $record) {
      $data = unserialize($record->data);
      $row[] = array('data' => l($record->sku, 'admin/commerce/products/' . $product->product_id, array('attributes' => array('absolute' => TRUE))), 'rowspan' => count($data));
      foreach ($data as $error) {
        $row[] = $error['field'];
        $row[] = $error['message'];
        $rows[] = array('data' => $row);
        $row = array();
      }
    }

    $content['information'] = array(
      '#markup' => '<p>' . t('This product is not exportable. Find here the list of values to complete in order to export your product.') . '</p>',
    );

    // Create the page.
    $content['pager_table'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t("There are no error(s) associated with this product."),
    );

    $content['pager_pager'] = array('#theme' => 'pager');
  }
  else {
    drupal_set_message(t('A product with SKU %sku could not be found. Please check that the product exists or import it first.', array('%sku' => $product_sku)), 'error');
  }

  return $content;
}

/**
 * Form callback : Add or edit a feed.
 *
 * @param $form
 * @param $form_state
 * @param CommercePriceministerFeed $feed
 *   A feed object
 *
 * @return mixed
 */
function commerce_priceminister_feed_edit_form($form, &$form_state, CommercePriceministerFeed $feed) {
  $form_state['feed'] = $feed;

  $form = array();
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $feed->title,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['local_product_type'] = array(
    '#type' => 'select',
    '#title' => t('Drupal Commerce products types'),
    '#options' => commerce_product_type_get_name(),
    '#description' => t('Select the product type on your store you want to export to Priceminister'),
    '#default_value' => $feed->local_product_type,
    '#required' => TRUE,
  );

  $form['local_product_display'] = array(
    '#type' => 'select',
    '#title' => t('Drupal Commerce products display'),
    '#options' => array('' => t('- Select -')) + commerce_priceminister_get_local_product_display(),
    '#description' => t('Select the product display linked to you product'),
    '#default_value' => $feed->local_product_display,
  );


  $form['priceminister_product_type'] = array(
    '#type' => 'select',
    '#title' => t('Priceminister products types'),
    '#options' => commerce_priceminister_get_priceminister_product_types(),
    '#description' => t('Select the Priceminister product type you want to map with your store product types'),
    '#default_value' => $feed->priceminister_product_type,
    '#required' => TRUE,
  );
  $form['frequency'] = array(
    '#type' => 'select',
    '#title' => t('Frequency export'),
    '#description' => t('Select the frequency to export yours products to Priceminister.<br /> You will have to enable the cron functionally on your Drupal installation. <a href="@cron-link">https://drupal.org/cron</a>', array('@cron-link' => url('https://drupal.org/cron'))),
    '#options' => array(
      '1' => t('every hours'),
      '2' => t('every 2 hours'),
      '3' => t('every 4 hours')
    ),
    '#default_value' => $feed->frequency,
  );
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#options' => array(0 => t('Disable'), 1 => t('Enable')),
    '#description' => t('Choose if the feed is enable or not.'),
    '#default_value' => $feed->status,
    '#required' => TRUE,
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => array('commerce_priceminister_feed_edit_delete_form_submit'),
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 5,
  );

  return $form;
}

/**
 * Submit handler for commerce_priceminister_feed_edit_form().
 */
function commerce_priceminister_feed_edit_form_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  $values = $form_state['values'];

  $feed->title = $values['title'];
  $feed->frequency = $values['frequency'];
  $feed->status = $values['status'];
  $feed->local_product_type = $values['local_product_type'];
  $feed->local_product_display = $values['local_product_display'];
  $feed->priceminister_product_type = $values['priceminister_product_type'];

  commerce_priceminister_feed_save($feed);
  $form_state['redirect'] = url('admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/map', array('absolute' => TRUE));
}

/**
 * Submit function: handle the 'Delete' button on the feed form.
 */
function commerce_priceminister_feed_edit_delete_form_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  $destination = drupal_get_destination();
  $form_state['redirect'] = array('admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/delete', array('query' => $destination));
}

/**
 * Generate the form to display fields that are already mapped.
 *
 * @param $form
 * @param $form_state
 * @param CommercePriceministerFeed $feed
 *   A feed object
 * @return array
 */
function commerce_priceminister_feed_mapped_fields_feed_form($form, &$form_state, $feed) {
  $form_state['feed'] = $feed;
  $rows = array();

  $form = array();
  $form['#mappings'] = array();
  $form['remove_flags'] = array('#tree' => TRUE);

  // Get the list a field that are mandatory for all imports.
  $hidden_fields = array_keys(CommercePriceministerFeed::getRequiredHiddenFields());

  // Generates the list of field that are mapped for this feed.
  foreach ($feed->fields_mapping as $mapping) {
    $local = $mapping['local'];
    // Extract information stored in the local value (either a field or a value).
    if (isset($local['field'])) {
      $entity_info = entity_get_info(key($local['field']));
      $field_local = t('@entity : @field', array('@entity' => $entity_info['label'], '@field' => reset($local['field'])));
    }
    elseif (isset($local['value'])) {
      $field_local = t('Default value : @value', array('@value' => $local['value']));
    }

    $priceminister = $mapping['priceminister']['field'];

    if (isset($priceminister)) {
      // Display the Drupalcommerce and Priceminister names for the field.
      $form['#mappings'][$priceminister['key']] = array(
        'field_local' => $field_local,
        'field_priceminister' => $priceminister['label'],
      );

      // Display info if the field is not a field mandatory for the system.
      if (!in_array($priceminister['key'], $hidden_fields)) {
        // Displays if the field is required or not and.
        $form['#mappings'][$priceminister['key']]['field_required'] = $priceminister['mandatory'];
        // Allow the merchant to remove the field from the feed.
        $form['remove_flags'][$priceminister['key']] = array(
          '#type' => 'checkbox',
          '#title' => t('Remove'),
          '#prefix' => '<div class="commerce-priceminister-remove">',
          '#suffix' => '</div>',
        );
      }
    }
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#prefix' => '<div class="mapped-fields">',
    '#suffix' => '</div>',
    '#value' => t('Submit'),
  );

  return $form;
}

/**
 * Submit handler for commerce_priceminister_feeds_feed_remove_field_mapping_submit().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_mapped_fields_feed_form_submit($form, &$form_state) {
  //form_state_values_clean($form_state);
  $removed_fields = array();
  $feed = $form_state['feed'];

  $priceminister_product_type_fields = $feed->getPriceministerProductTypeFields();

  // Marks fields that doesn't exist in the product type fields definition.
  // This could happens if Priceminister changed fields definition.
  $removed_fields += array_diff_key($feed->fields_mapping, $priceminister_product_type_fields);

  // Removed selected fields by the user.
  $removed_fields += array_filter($form_state['values']['remove_flags']);

  foreach ($removed_fields as $key => $value) {
    unset($feed->fields_mapping[$key]);
  }
  commerce_priceminister_feed_save($feed);
}

/**
 * Theme function to display mapped fields of a feed.
 *
 * @param array $variables
 *
 * @return string
 */
function theme_commerce_priceminister_feed_mapped_fields_feed_form($variables) {
  $form = $variables['form'];
  $rows = array();

  $header = array(
    array('data' => t('Priceminister field')),
    array('data' => t('Local field')),
    array('data' => t('Required')),
    array('data' => '&nbsp;', 'class' => 'action'),
  );

  // Generates the table content of mapped fields.
  foreach ($form['#mappings'] as $i => $mapping) {
    // Display the status of the field for the feed, mandatory, required or not
    // required
    $required = NULL;
    if (isset($mapping['field_required'])) {
      $required = $mapping['field_required'] == TRUE ? t('Yes') : t('No');
    }
    $rows[] = array(
      'data' => array(
        check_plain($mapping['field_priceminister']),
        check_plain($mapping['field_local']),
        $required,
        drupal_render($form['remove_flags'][$i]),
      ),
    );
  }

  if (!count($rows)) {
    $rows[] = array(
      array(
        'colspan' => 4,
        'data' => t('No mappings defined.'),
      ),
    );
  }

  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'commerce-priceminister-mapping-overview')));
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * Generate an overview of field that have already been mapped
 * The form display mapped fields between the website and Priceminister. The
 * user has the possibility to remove mapped field from this form.
 *
 * @param $form
 * @param $form_state
 * @param CommercePriceministerFeed $feed
 *   A feed object
 * @param $priceminister_field
 * @param array $local_product_type_fields_widget_options
 * @param array $priceminister_product_type_fields_widget_options
 * @return mixed
 */
function commerce_priceminister_feed_add_field_form($form, &$form_state, $feed, $priceminister_product_type_fields_widget_options = array(), $local_product_type_fields_widget_options = array(), $priceminister_field = array()) {
  $form_state['feed'] = $feed;
  // Generate a unique field key in order to let JQuery access to a special
  // field.
  $unique_field_key = isset($priceminister_field['key']) ? '|' . $priceminister_field['key'] : '';

  $form['priceminister' . $unique_field_key] = array(
    '#type' => 'select',
    '#default_value' => !empty($priceminister_field)
      ? $priceminister_field['key']
      : '',
    '#description' => !empty($priceminister_field['description'])
      ? t('@description', array('@description' => $priceminister_field['description']))
      : '',
    '#options' => empty($priceminister_field)
      ? array('' => t('Select a Priceminister\'s field')) + $priceminister_product_type_fields_widget_options
      : array($priceminister_field['key'] => $priceminister_field['label']),
    '#disabled' => empty($priceminister_field) ? FALSE : TRUE,
    '#prefix' => '<div class="pm_field">',
    "#suffix" => '</div>',
  );

  // Add the other possibility to let the merchant fill in a default value.
  $local_product_type_fields_widget_options += array(t('Other') => array('other' => t('Add a default value')));

  $form['select' . $unique_field_key] = array(
    '#type' => 'select',
    '#options' => array('' => t('Select a local field')) + $local_product_type_fields_widget_options,
    '#prefix' => '<div class="local_field">',
  );

  $form['textfield' . $unique_field_key] = array(
    '#type' => 'textfield',
    '#description' => t('This value will be assigned to each products sent to Priceminister.'),
    '#suffix' => '</div>',
    '#states' => array(
      'visible' => array(
        ':input[name="select' . $unique_field_key . '"]' => array('value' => 'other'),
      ),
    ),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
    '#validate' => array('commerce_priceminister_feeds_feed_add_field_mapping_validate'),
    '#submit' => array('commerce_priceminister_feeds_feed_add_field_mapping_submit'),
    '#prefix' => '<div class="action">',
    '#suffix' => '</div>',
  );

  $form['#attached']['css'][] = drupal_get_path('module', 'commerce_priceminister') . '/css/commerce_priceminister.css';

  return $form;
}

/**
 * Validate handler for commerce_priceminister_feed_add_field_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feeds_feed_add_field_mapping_validate($form, &$form_state) {
  $fields = array();
  foreach ($form_state['values'] as $field_form_key => $value) {
    list($key) = explode('|', $field_form_key);
    $fields[$key] = $field_form_key;
  }

  if ($form_state['values'][$fields['select']] == 'other' && empty($form_state['values'][$fields['textfield']])) {
    form_set_error($fields['textfield'], t('You choosed to add a default value. You must fill in value in the field.'));
  }
}

/**
 * Submit handler for commerce_priceminister_feed_add_field_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feeds_feed_add_field_mapping_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  form_state_values_clean($form_state);

  // Regenerates a normed array of values compound with fields values. This is
  // done to easily access to values without using their complex real names.
  $values = array();
  foreach ($form_state['values'] as $key => $value) {
    list($new_key) = explode('|', $key);
    $values[$new_key] = $value;
  }

  // Generates a mapping array to save it into the feed. Each mapping has the
  // same structure, a Priceminister field and a local field. The local field
  // can be compound with a real field or a default value.
  // Here's an example for a field:
  if ($feed && $values) {
    // If the user has chooses to add a value.
    if ($values['select'] == 'other' && (!empty($values['textfield']))) {
      $local = array('value' => $values['textfield']);
    }
    // If the user has chooses to add a value.
    elseif (!empty($values['select'])) {
      list($entity, $field_key) = explode('|', $values['select']);
      $local = array(
        'field' => array(
          $entity => $field_key
        ),
      );
    }
  }

  $priceminister_field = $feed->getPriceministerProductTypeField($values['priceminister']);

  if ($values['priceminister'] && $local) {
    $feed->fields_mapping[$values['priceminister']] = array(
      'local' => $local,
      'priceminister' => array(
        'field' => $priceminister_field
      ),
    );
  }

  commerce_priceminister_feed_save($feed);
  drupal_set_message(t('The feed mapping configuration has been saved.'));

  if ($products = commerce_product_load_multiple(array(), array('type' => $feed->local_product_type))) {
    $products_queue = DrupalQueue::get('priceminister_product_feed_' . $feed->id);
    foreach ($products as $product) {
      if (_commerce_priceminister_is_product_exportable($product->product_id, $feed->id)) {
        $products_queue->createItem($product->sku);
      }
    }
  }
}

/**
 * Form callback : Delete a feed.
 *
 * @param $form
 * @param $form_state
 * @param CommercePriceministerFeed $feed
 *   A feed object
 *
 * @return mixed
 */
function commerce_priceminister_feed_delete_form($form, &$form_state, CommercePriceministerFeed $feed) {
  $form_state['feed'] = $feed;

  $destination = array();
  if (isset($_GET['destination'])) {
    $path = drupal_get_destination();
    $path = $path['destination'];
    unset($_GET['destination']);
  }
  else {
    $path = url('admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id, array('absolute' => TRUE));
  }

  return confirm_form($form,
    t('Are you sure you want to delete %feed?', array('%feed' => $feed->title)),
    $path,
    t('Your are going to delete the feed named: %feed_title. This action cannot be undone.', array('%feed_title' => $feed->title)),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Submit handler for commerce_marketplace_feed_delete_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_delete_form_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  if ($feed) {
    commerce_priceminister_feed_delete($feed->id);
    drupal_set_message(t('The feed has been deleted.'));
  }
  $form_state['redirect'] = url('admin/commerce/config/marketplaces/priceminister/feeds', array('absolute' => TRUE));
}

/**
 * Form callback : Exports products for a feed.
 *
 * @param $form
 * @param $form_state
 * @param CommercePriceministerFeed $feed
 *   A feed object
 */
function commerce_priceminister_feed_export_form($form, &$form_state, CommercePriceministerFeed $feed) {
  $form_state['feed'] = $feed;
  $products_queue = DrupalQueue::get('priceminister_product_feed_' . $feed->id);
  $number_of_products_to_export = $products_queue->numberOfAvailableItems();

  $question = t('Export: %feed feed', array('%feed' => $feed->title));
  if ($number_of_products_to_export > 0) {
    $description = t('Your are going to export %number products.', array('%number' => $products_queue->numberOfAvailableItems()));
  }
  else {
    $description = t('There is nothing to export for this feed.');
  }
  $description = '<p>' . $description . '<br /><br />' . t('Find the list of available products on the <a href="@product-page-link">product page</a> of your feed.', array('@product-page-link' => url('admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id . '/product'))) . '</p>';
  $path = 'admin/commerce/config/marketplaces/priceminister/feed/' . $feed->id;

  $form = confirm_form($form, $question, $path, $description);
  $form['actions']['cancel']['#access'] = FALSE;
  if ($number_of_products_to_export <= 0) {
    $form['actions']['submit']['#disabled'] = 'disabled';
  }

  return $form;
}

/**
 * Submit handler for commerce_priceminister_feed_export_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_export_form_submit($form, &$form_state) {
  if ($feed = $form_state['feed']) {
    module_load_include('inc', 'commerce_priceminister', 'includes/commerce_priceminister.export');
    commerce_priceminister_export_feed_batch($feed);
  }
}

/**
 * Form callback : Add and configure feed form.
 *
 * Display fields that are mandatory and that are already mapped.
 * Allow the merchant to add fields (mandatory or not) to a field.
 *
 * @param $feed
 * @return array
 */
function commerce_priceminister_feed_fields_mapping($feed) {
  $page = array();

  // Displays fields that are alreay mapped.
  $page['mapped_fields'] = drupal_get_form('commerce_priceminister_feed_mapped_fields_feed_form', $feed);
  $page['mapped_fields']['#prefix'] = '<h2>' . t('Mapped fields') . '</h2>';

  // Displays the form to allow the merchant to map required fields.
  $page['required_fields'] = drupal_get_form('commerce_priceminister_feed_required_fields_feed_form', $feed);
  $page['required_fields']['#prefix'] = '<h2>' . t('Required fields') . '</h2>';

  // Adds the form to allow the merchant to map optional fields.
  $page['optional_fields'] = drupal_get_form('commerce_priceminister_feed_optional_fields_feed_form', $feed);
  $page['optional_fields']['#prefix'] = '<h2>' . t('Optional fields') . '</h2>';

  return $page;
}

/**
 * Form callback : Allows to map required fields to a feed.
 *
 * Displays a select list of fields from Priceminister to map with all fields
 * available on Drupal Commernce.
 *
 * @param $form
 * @param $form_state
 * @param $feed
 *
 * @return array
 */
function commerce_priceminister_feed_required_fields_feed_form($form, &$form_state, $feed) {
  $form_state['feed'] = $feed;

  // Gets the list of fields that are already mapped.
  $mapped_fields = $feed->getPriceministerProductTypeMappedFields(TRUE);

  // Gets the list of fields that are required for this type of feed.
  $priceminister_product_fields = array_diff_key($feed->getPriceministerProductTypeFields(array('required' => TRUE)), $mapped_fields);

  // Gets the list of fields available on Drupal Commerce.
  $local_product_type_fields_widget_options = _commerce_priceminister_get_local_product_type_field_widget_option($feed);

  $form['required_fields'] = array(
    '#tree' => TRUE,
  );

  // Generates for each required field a select list with the available fields
  // and a textfield to let the merchant fill in a default value.
  foreach ($priceminister_product_fields as $field_name => $field_properties) {
    $field = $field_properties['key'];

    // Add the other possibility to let the merchant fill in a default value.
    $local_product_type_fields_widget_options += array(t('Other') => array('other' => t('Add a default value')));

    $form['required_fields'][$field] = array(
      'priceminister|' . $field => array(
        '#type' => 'markup',
        '#markup' => ucfirst($field_properties['label']),
        '#prefix' => '<div class="priceminister-field">',
        "#suffix" => '</div>',
      ),
      'select|' . $field => array(
        '#type' => 'select',
        '#options' => array('' => t('Select a field')) + $local_product_type_fields_widget_options,
        '#default_value' => 'commerce_priceminister_product_metadata|' . $field,
        '#prefix' => '<div class="drupal-commerce-optional-field">',
      ),
      'textfield|' . $field => array(
        '#type' => 'textfield',
        '#description' => t('This value will be assigned to each products sent to Priceminister.'),
        '#suffix' => '</div>',
        '#states' => array(
          'visible' => array(
            ':input[name="required_fields[' . $field . '][select|' . $field . ']"]' => array('value' => 'other'),
          ),
        ),
      ),
    );
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#prefix' => '<div class="required-fields">',
    '#suffix' => '</div>',
    '#value' => t('Submit'),
  );

  return $form;
}

/**
 * Form validate for commerce_priceminister_feed_required_fields_feed_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_required_fields_feed_form_validate($form, &$form_state) {
  // Parses all the field in order to detect if one of them is supposed to be
  // a value filled in by the merchant.
  foreach ($form_state['values']['required_fields'] as $field_name => $field_value) {
    if ($field_value["select|{$field_name}"] == 'other' && empty($field_value["textfield|{$field_name}"])) {
      // Gets the text displayed for the priceminister field.
      $priceminister_field = $form['required_fields'][$field_name]["priceminister|{$field_name}"]['#markup'];
      // Displays an error message to the merchant.
      form_set_error("required_fields][{$field_name}][textfield|{$field_name}", t('You choosed to add a default value for this field: %field_name, but you have to fill in a value.', array('%field_name' => $priceminister_field)));
    }
  }
}

/**
 * Form submit for commerce_priceminister_feed_required_fields_feed_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_required_fields_feed_form_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  // Cleans unusefull values in the form_state.
  form_state_values_clean($form_state);

  // Adds new fields to the feed.
  foreach ($form_state['values']['required_fields'] as $field_name => $field_value) {
    $drupal_commerce_field = array();
    if (!empty($field_value["select|{$field_name}"])) {
      if ($field_value["select|{$field_name}"] == 'other') {
        // Gets the value from the textfield
        $drupal_commerce_field = array('value' => $field_value["textfield|{$field_name}"]);
      }
      else {
        // Gets the value from the selectlist.
        list($entity, $field_key) = explode('|', $field_value["select|{$field_name}"]);
        $drupal_commerce_field = array(
          'field' => array(
            $entity => $field_key
          ),
        );
      }
    }

    // Adds the new field to the feed object in order to save it.
    if ($drupal_commerce_field) {
      $priceminister_field = $feed->getPriceministerProductTypeField($field_name);
      $feed->fields_mapping[$field_name] = array(
        'local' => $drupal_commerce_field,
        'priceminister' => array(
          'field' => $priceminister_field
        ),
      );
    }
  }

  // Save the feed.
  commerce_priceminister_feed_save($feed);
  drupal_set_message(t('The feed mapping configuration has been saved.'));

  // Push the feed into the queue to be (re)exported.
  if ($products = commerce_product_load_multiple(array(), array('type' => $feed->local_product_type))) {
    $products_queue = DrupalQueue::get('priceminister_product_feed_' . $feed->id);
    foreach ($products as $product) {
      if (_commerce_priceminister_is_product_exportable($product->product_id, $feed->id)) {
        $products_queue->createItem($product->sku);
      }
    }
  }
}

/**
 * Form theme function for commerce_priceminister_feed_required_fields_feed_form().
 *
 * Change the display of the form to a theme table
 *
 * @param $variables
 * @return string
 */
function theme_commerce_priceminister_feed_required_fields_feed_form($variables) {
  $form = $variables['form'];
  $rows = array();
  $rows = array();
  foreach (element_children($form['required_fields']) as $field) {
    // Check if is numeric ensures that only the data fields are rendered as a rows.  This prevents things like submit buttons, etc being added to table rows.
      $row = array();
      $row[] = drupal_render($form['required_fields'][$field]['priceminister|' . $field]);
      $row[] = drupal_render($form['required_fields'][$field]['select|' . $field]) . drupal_render($form['required_fields'][$field]['textfield|' . $field]);
      $rows[] = $row;
  }

  // Define the table headers.
  $header = array(
    t('Priceminister field'),
    t('Drupal commerce field'),
  );

  $output = '';
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'commerce-priceminister-mapping-overview')));
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * Form callback : Allows to map optional fields to a feed.
 *
 * Displays a select list of fields from Priceminister to map with all fields
 * available on Drupal Commernce.
 *
 * @param $form
 * @param $form_state
 * @param $feed
 *
 * @return array
 */
function commerce_priceminister_feed_optional_fields_feed_form($form, &$form_state, $feed) {
  $form_state['feed'] = $feed;
  $rows = array();
  $form = array();

  // Gets the list of fields that are already mapped.
  $mapped_fields = $feed->getPriceministerProductTypeMappedFields(TRUE);

  // Gets the list of fields that are NOT required for this type of feed.
  $priceminister_product_fields = array_diff_key($feed->getPriceministerProductTypeFields(array('required' => FALSE)), $mapped_fields);

  // Gets the list of fields available on Drupal Commerce.
  $local_product_type_fields_widget_options = _commerce_priceminister_get_local_product_type_field_widget_option($feed);

  // Generate a field_id => field_label pair array of source fields.
  $priceminister_product_type_optional_fields_widget_options = array();
  foreach ($priceminister_product_fields as $key => $field) {
    $priceminister_product_type_optional_fields_widget_options[$key] = $field['label'];
  }

  // Add the other possibility to let the merchant fill in a default value.
  $local_product_type_fields_widget_options += array(t('Other') => array('other' => t('Add a default value')));

  $form['priceminister'] = array(
      '#type' => 'select',
      '#options' => array('' => t('Select a Priceminister\'s field')) + $priceminister_product_type_optional_fields_widget_options,
      '#prefix' => '<div class="priceminister-field">',
      "#suffix" => '</div>',
  );
  $form['select'] = array(
    '#type' => 'select',
    '#options' => array('' => t('Select a field')) + $local_product_type_fields_widget_options,
    '#prefix' => '<div class="drupal-commerce-optional-field">',
  );
  $form['textfield'] = array(
    '#type' => 'textfield',
    '#description' => t('This value will be assigned to each products sent to Priceminister.'),
    '#suffix' => '</div>',
    '#states' => array(
      'visible' => array(
        ':input[name="select"]' => array('value' => 'other'),
      ),
    ),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#prefix' => '<div class="required-fields">',
    '#suffix' => '</div>',
    '#value' => t('Submit'),
  );

  return $form;
}

/**
 * Form validate for commerce_priceminister_feed_optional_fields_feed_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_optional_fields_feed_form_validate($form, &$form_state) {
  $feed = $form_state['feed'];

  // Checks if the priceminister field is not empty.
  if (empty($form_state['values']['priceminister'])) {
    form_set_error("priceminister", t('You have to choose a field to map on Priceminister'));
  }
  if ($form_state['values']['select'] == 'other' && empty($form_state['values']['textfield'])) {
    // Gets the text displayed for the priceminister field.
    $priceminister_field = $form['priceminister']['#options'][$form_state['values']['priceminister']];
    // Displays an error message to the merchant.
    form_set_error("textfield", t('You choosed to add a default value for this field: %field_name, but you have to fill in a value.', array('%field_name' => $priceminister_field)));
  }
}

/**
 * Form submit for commerce_priceminister_feed_optional_fields_feed_form().
 *
 * @param $form
 * @param $form_state
 */
function commerce_priceminister_feed_optional_fields_feed_form_submit($form, &$form_state) {
  $feed = $form_state['feed'];
  form_state_values_clean($form_state);



  $drupal_commerce_field = array();
  if (!empty($form_state['values']["select"])) {
    if ($form_state['values']["select"] == 'other') {
      // Gets the value from the textfield
      $drupal_commerce_field = array('value' => $form_state['values']["textfield"]);
    }
    else {
      // Gets the value from the selectlist.
      list($entity, $field_key) = explode('|', $form_state['values']["select"]);
      $drupal_commerce_field = array(
        'field' => array(
          $entity => $field_key
        ),
      );
    }
  }

  // Adds the new field to the feed object in order to save it.
  if ($drupal_commerce_field) {
    $priceminister_field = $feed->getPriceministerProductTypeField($form_state['values']["priceminister"]);
    $feed->fields_mapping[$form_state['values']["priceminister"]] = array(
      'local' => $drupal_commerce_field,
      'priceminister' => array(
        'field' => $priceminister_field,
      ),
    );
  }

  commerce_priceminister_feed_save($feed);
  drupal_set_message(t('The feed mapping configuration has been saved.'));

  if ($products = commerce_product_load_multiple(array(), array('type' => $feed->local_product_type))) {
    $products_queue = DrupalQueue::get('priceminister_product_feed_' . $feed->id);
    foreach ($products as $product) {
      if (_commerce_priceminister_is_product_exportable($product->product_id, $feed->id)) {
        $products_queue->createItem($product->sku);
      }
    }
  }
}

/**
 * Form theme function for commerce_priceminister_feed_optional_fields_feed_form().
 *
 * Change the display of the form to a theme table
 *
 * @param $variables
 * @return string
 */
function theme_commerce_priceminister_feed_optional_fields_feed_form($variables) {
  $form = $variables['form'];

  // Define the table headers.
  $header = array(
    t('Priceminister field'),
    t('Drupal commerce field'),
  );

  $rows = array(
    array(
      drupal_render($form['priceminister']),
      drupal_render($form['select']) . drupal_render($form['textfield']),
    ),
  );

  $output = '';
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'commerce-priceminister-mapping-overview')));
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * Generates a list of fields available for a feed that can be contained either
 * in a Drupal commerce entity or in the product metadata or in the product
 * display.
 *
 * @param $feed
 * @return array
 */
function _commerce_priceminister_get_local_product_type_field_widget_option($feed) {
  $widget_option = &drupal_static(__FUNCTION__ . $feed->id);
  if (!isset($widget_option)) {
    // PRODUCT METADATA
    // Generate the list of available fields for the
    // priceminister_product_metadata entity.
    $product_metadata_fields = commerce_priceminister_get_product_metadata_fields($feed->priceminister_product_type);
    $fields = array();
    foreach ($product_metadata_fields as $field) {
      $fields['commerce_priceminister_product_metadata|' . $field['key']] = ucfirst($field['label']);
    }
    asort($fields);
    $widget_option = array(t('Additional product information') => $fields);

    // DRUPAL COMMERCE ENTITIES
    // Gets the list of fields available for this Drupal Commerce product.
    $product_type_fields = $feed->getEntityFields('commerce_product');

    // Generate a field_id => field_label pair array of source fields for the
    // commerce_product entity.
    $fields = array();
    foreach ($product_type_fields as $key => $field) {
      $fields['commerce_product|' . $key] = ucfirst($field['label']);
    }
    asort($fields);
    $widget_option += array(t('Product') => $fields);

    // PRODUCT DISPLAY
    // Get the list of available fields to the product display.
    $product_display_fields = $feed->getEntityFields('node');
    $fields = array();
    foreach ($product_display_fields as $key => $field) {
      $fields['commerce_priceminister_product_display|' . $key] = ucfirst($field['label']);
    }

    asort($fields);
    $widget_option += array(t('Product display') => $fields);
  }

  return $widget_option;
}